/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdarg.h>
#include <string.h>

#include <gtk/gtk.h>

#include "appdir.h"
#include "dialog.h"
#include "i18n.h"

static int
dialog_run(GtkWindow *parent_win, GtkDialogFlags flags,
        GtkMessageType mt, GtkButtonsType bt, const char *formatted_message)
{
    int result;
    GtkWidget *dialog = gtk_message_dialog_new(parent_win, flags, mt,
            bt, "%s", formatted_message);
            
    if (mt == GTK_MESSAGE_ERROR)
        gtk_window_set_title(GTK_WINDOW(dialog), _("Error from rox-media"));
    result = gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
    return result;
}

int
dialog_error(GtkWindow *parent_win, GtkDialogFlags flags,
        const char *message, ...)
{
    int result;
    va_list ap;
    char *formatted;
    
    va_start(ap, message);
    formatted = g_strdup_vprintf(message, ap);
    va_end(ap);
    result = dialog_run(parent_win, flags, GTK_MESSAGE_ERROR,
            GTK_BUTTONS_CLOSE, formatted);
    g_free(formatted);
    return result;
}
    
int
dialog_warning(GtkWindow *parent_win, GtkDialogFlags flags,
        const char *message, ...)
{
    int result;
    va_list ap;
    char *formatted;
    
    va_start(ap, message);
    formatted = g_strdup_vprintf(message, ap);
    va_end(ap);
    result = dialog_run(parent_win, flags, GTK_MESSAGE_WARNING,
            GTK_BUTTONS_CLOSE, formatted);
    g_free(formatted);
    return result;
}

char *
human_readable_size(double size, int mantissa)
{
    const char *suffixes = "KMGTPEZY";
    char *suffix;
    char *result;
    
    if (size > 9999)
        return human_readable_size(size / 1024, mantissa + 1);
    if (!mantissa)
        suffix = g_strdup("");
    else if (mantissa >= strlen(suffixes))
        suffix = g_strdup_printf("x10^%d", mantissa * 3);
    else
        suffix = g_strdup_printf("%c", suffixes[mantissa - 1]);
    if (size > 999)
        result = g_strdup_printf("%d%s", (int) size, suffix);
    else if (size > 99)
        result = g_strdup_printf("%.1f%s", size, suffix);
    else if (size > 9)
        result = g_strdup_printf("%.2f%s", size, suffix);
    else
        result = g_strdup_printf("%.3f%s", size, suffix);
    g_free(suffix);
    return result;
}
    
struct _PropertiesDialog {
    GtkWidget *dialog;
    GduPresentable *pres;
    GduDevice *dev;
    GtkWidget *icon;
    GtkWidget *name;
    GtkWidget *vpd_name;
    GtkWidget *desc;
    GtkWidget *size;
    GtkWidget *mount_point;
};

GtkWidget *
dialog_properties_get_widget(PropertiesDialog *pd)
{
    return pd->dialog;
}

void
dialog_properties_update(PropertiesDialog *pd)
{
    GIcon *gicon = gdu_presentable_get_icon(pd->pres);
    char *name = gdu_presentable_get_name(pd->pres);
    char *markup = g_strdup_printf("<b>%s</b>", name);
    char *vpd_name = gdu_presentable_get_vpd_name(pd->pres);
    char *desc = gdu_presentable_get_description(pd->pres);
    
    gtk_image_set_from_gicon(GTK_IMAGE(pd->icon), gicon, GTK_ICON_SIZE_DIALOG),
    g_object_unref(gicon);
    
    gtk_label_set_markup(GTK_LABEL(pd->name), markup);
    g_free(markup);
    g_free(name);
    
    gtk_label_set_text(GTK_LABEL(pd->vpd_name), vpd_name);
    g_free(vpd_name);
    
    gtk_label_set_text(GTK_LABEL(pd->desc), desc);
    g_free(desc);
    
    markup = human_readable_size(
            (double) gdu_presentable_get_size(pd->pres), 0);
    gtk_label_set_text(GTK_LABEL(pd->size), markup);
    g_free(markup);
    
    if (gdu_device_is_mounted(pd->dev))
    {
        gtk_label_set_text(GTK_LABEL(pd->mount_point),
                gdu_device_get_mount_path(pd->dev));
    }
    else
    {
        gtk_label_set_text(GTK_LABEL(pd->mount_point),
                _("Not mounted"));
    }
}

PropertiesDialog *
dialog_properties_new(GduPresentable *pres, GduDevice *dev)
{
    PropertiesDialog *pd = g_new(PropertiesDialog, 1);
    GtkWidget *table = gtk_table_new(5, 2, FALSE);
    char *win_title = g_strdup_printf(_("%s properties"),
            gdu_device_get_device_file(dev));
    const GtkAttachOptions a = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
    const int p = 4;
    GtkWidget *label;
    
    pd->dialog = gtk_dialog_new_with_buttons(win_title, NULL, 0,
            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
    g_free(win_title);
    
    pd->pres = pres;
    pd->dev = dev;
    
    pd->icon = gtk_image_new();
    gtk_table_attach(GTK_TABLE(table), pd->icon,
            0, 1, 0, 1, GTK_SHRINK, a, p, p);
    
    pd->name = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(pd->name), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), pd->name,
            1, 2, 0, 1, a, a, p, p);
    
    label = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_markup(GTK_LABEL(label), "<b>Full name:</b>");
    gtk_table_attach(GTK_TABLE(table), label,
            0, 1, 1, 2, a, a, p, p);
    pd->vpd_name = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(pd->vpd_name), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), pd->vpd_name,
            1, 2, 1, 2, a, a, p, p);
    
    label = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_markup(GTK_LABEL(label), "<b>Description:</b>");
    gtk_table_attach(GTK_TABLE(table), label,
            0, 1, 2, 3, a, a, p, p);
    pd->desc = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(pd->desc), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), pd->desc,
            1, 2, 2, 3, a, a, p, p);
    
    label = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_markup(GTK_LABEL(label), "<b>Size:</b>");
    gtk_table_attach(GTK_TABLE(table), label,
            0, 1, 3, 4, a, a, p, p);
    pd->size = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(pd->size), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), pd->size,
            1, 2, 3, 4, a, a, p, p);
    
    label = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_markup(GTK_LABEL(label), "<b>Mount point:</b>");
    gtk_table_attach(GTK_TABLE(table), label,
            0, 1, 4, 5, a, a, p, p);
    pd->mount_point = gtk_label_new(NULL);
    gtk_misc_set_alignment(GTK_MISC(pd->mount_point), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), pd->mount_point,
            1, 2, 4, 5, a, a, p, p);
    
    dialog_properties_update(pd);
    
    gtk_container_add(
            GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(pd->dialog))),
            table);
    gtk_widget_show_all(table);
    return pd;
}



void
dialog_about(void)
{
    GtkWidget *w = gtk_about_dialog_new();
    GtkAboutDialog *d = GTK_ABOUT_DIALOG(w);
    const char *authors[] = { "Tony Houghton <h@realh.co.uk", NULL };
    GdkPixbuf *logo;
    
    gtk_about_dialog_set_program_name(d, "rox-media");
    gtk_about_dialog_set_version(d, VERSION_STRING);
    gtk_about_dialog_set_copyright(d, "\302\251 2009 Tony Houghton");
    gtk_about_dialog_set_comments(d,
            _("Manage removable drives and discs for ROX"));
    gtk_about_dialog_set_authors(d, authors);
    logo = gdk_pixbuf_new_from_file(appdir_get_app_icon(), NULL);
    if (logo)
    {
        gtk_about_dialog_set_logo(d, logo);
        g_object_unref(logo);
    }
    gtk_dialog_run(GTK_DIALOG(w));
    gtk_widget_destroy(w);
}
