/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
       
#include <sys/types.h>
#include <unistd.h>

#include <errno.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib/gstdio.h>
#include <gdu/gdu.h>
#include <gdk/gdk.h>
#include <gconf/gconf-client.h>

#include "appdir.h"
#include "dialog.h"
#include "i18n.h"
#include "icon_group.h"
#include "icon_source_emblem.h"
#include "icon_source_filename.h"
#include "icon_source_gicon.h"
#include "panel_icon.h"

typedef struct {
    IconGroup *group;
    GdkScreen *screen;
    HPanelIcon *main_icon;
    gboolean systray;
    GduPool *pool;
    //GList *added_devices;
} RoxMedia;

typedef struct _AddedDeviceData AddedDeviceData;
typedef struct _OpticalDriveData OpticalDriveData;

typedef enum {
    EMBLEM_NONE,
    EMBLEM_MOUNTED,
    EMBLEM_MOUNTING
} EmblemStatus;

struct _AddedDeviceData {
    RoxMedia *media;
    /* NB presentable and dev are apparently invalid
     * during "changed" and "removed" signals
     */
    GduPresentable *presentable;
    GduDevice *dev;
    HPanelIcon *panel_icon;
    gulong removed_tag, changed_tag;
    OpticalDriveData *odd;
    HIconSourceEmblem *icon_source;
    gboolean mounting;
    EmblemStatus emblem_status;
    GtkWidget *menu;
    gboolean open_when_mounted;
    PropertiesDialog *pd;
    GduDevice *ejectee;
};

struct _OpticalDriveData {
    AddedDeviceData *disc_add;
    GduDevice *dev;
    gulong removed_tag, changed_tag;
};


static gboolean disable_emblems = FALSE;


static void
close_menu(AddedDeviceData *add)
{
    if (add->menu)
    {
        gtk_widget_destroy(add->menu);
        add->menu = NULL;
    }
}

static void
set_tooltip(AddedDeviceData *add)
{
    char *devmount;
    char *tooltip;
    GduDevice *dev = add->dev;
    const char *label = NULL;
    char *pres_name = gdu_presentable_get_name(add->presentable);
    
    if (gdu_device_is_mounted(dev))
    {
        devmount = g_strdup_printf(_("<b>%s</b> mounted at %s"),
                gdu_device_get_device_file(dev),
                gdu_device_get_mount_path(dev));
    }
    else
    {
        devmount = g_strdup_printf(_("<b>%s</b>"),
                gdu_device_get_device_file(dev));
    }
    if (gdu_device_is_partition(dev))
        label = gdu_device_partition_get_label(dev);
    if (label && label[0])
        tooltip = g_strdup_printf("%s\n'%s'\n%s", devmount, label, pres_name);
    else
        tooltip = g_strdup_printf("%s\n%s", devmount, pres_name);
    g_free(devmount);
    g_free(pres_name);
    h_panel_icon_set_tooltip_markup(add->panel_icon, tooltip);
    g_free(tooltip);
}

static void
disconnect_odd_handlers(OpticalDriveData *odd)
{
    if (odd->removed_tag)
    {
        g_signal_handler_disconnect(odd->dev, odd->removed_tag);
        odd->removed_tag = 0;
    }
    if (odd->changed_tag)
    {
        g_signal_handler_disconnect(odd->dev, odd->changed_tag);
        odd->changed_tag = 0;
    }
}

static void
device_removed(GduDevice *dev, AddedDeviceData *add)
{
    OpticalDriveData *odd = add->odd;
    
    if (add->removed_tag)
    {
        g_signal_handler_disconnect(dev, add->removed_tag);
        add->removed_tag = 0;
    }
    if (add->changed_tag)
    {
        g_signal_handler_disconnect(dev, add->changed_tag);
        add->changed_tag = 0;
    }
    if (add->pd)
    {
        gtk_widget_destroy(dialog_properties_get_widget(add->pd));
    }
    if (odd)
    {
        disconnect_odd_handlers(odd);
        if (odd->dev)
        {
            g_object_unref(odd->dev);
            odd->dev = NULL;
        }
        g_free(odd);
    }
    close_menu(add);
    //add->media->added_devices = g_list_remove(add->media->added_devices, add);
    h_panel_icon_destroy(add->panel_icon);
    /* This wil also destroy its HIconSource */
    g_free(add);
}

static void
show_mount_status(AddedDeviceData *add)
{
    static GdkPixbuf *mounted_emblem = NULL;
    static GdkPixbuf *mounting_emblem = NULL;
    GdkPixbuf **emblem;
    const char *leafname;
    EmblemStatus status;
    
    if (disable_emblems)
        return;
    if (add->mounting)
    {
        emblem = &mounting_emblem;
        leafname = "mounting.png";
        status = EMBLEM_MOUNTING;
    }
    else if (gdu_device_is_mounted(add->dev))
    {
        emblem = &mounted_emblem;
        leafname = "mounted.png";
        status = EMBLEM_MOUNTED;
    }
    else
    {
        emblem = NULL;
        leafname = NULL;
        status = EMBLEM_NONE;
    }
    if (status == add->emblem_status)
        return;
    add->emblem_status = status;
    if (emblem)
    {
        if (!*emblem)
        {
            GError *error = NULL;
            char *emblem_file = appdir_get_image(leafname);
            
            *emblem = gdk_pixbuf_new_from_file(emblem_file, &error);
            if (!*emblem)
            {
                g_warning(_("Unable to load emblem file '%s': %s"),
                        emblem_file, error->message);
                g_error_free(error);
            }
        }
        if (*emblem)
            h_icon_source_emblem_set_emblem(add->icon_source, *emblem);
    }
    h_icon_source_emblem_set_show_emblem(add->icon_source,
            status != EMBLEM_NONE);
}

static void
device_changed(GduDevice *dev, AddedDeviceData *add)
{
    if (gdu_device_is_optical_disc(dev) && !gdu_device_is_media_available(dev))
    {
        device_removed(dev, add);
        return;
    }
    if (add->pd)
        dialog_properties_update(add->pd);
    close_menu(add);
    show_mount_status(add);
}

static void
parent_removed(GduDevice *dev, OpticalDriveData *odd)
{
    if (!gdu_device_is_media_available(dev))
    {
        AddedDeviceData *add = odd->disc_add;
        
        disconnect_odd_handlers(odd);
        /* I think dev gets unrefed automatically */
        g_free(odd);
        add->odd = NULL;
        device_removed(add->dev, add);
    }
}

static void
parent_changed(GduDevice *dev, OpticalDriveData *odd)
{
    if (!gdu_device_is_media_available(dev))
    {
        device_removed(odd->disc_add->dev, odd->disc_add);
    }
}

static gboolean
filer_run(const char *command, const char *mount_point, GError **error)
{
    char *full_command = strstr(command, "%s") ?
            g_strdup_printf(command, mount_point) :
            g_strdup_printf("%s %s", command, mount_point);
    gboolean result = g_spawn_command_line_async(full_command, error);
    
    g_free(full_command);
    return result;
}

static void
filer_open(AddedDeviceData *add, const char *mount_point)
{
    GError *error = NULL;
    char *command = gconf_client_get_string(gconf_client_get_default(),
            "/apps/rox-media/filer_open_command", &error);
    
    if (!command && !error)
        command = g_strdup("rox");
    if (command && !command[0])
    {
        g_free(command);
        command = g_strdup("rox");
    }

    if (error || !filer_run(command, mount_point, &error))
    {
        dialog_error(icon_group_get_window(add->media->group), 0,
                _("Unable to open mount point '%s': %s"),
                mount_point, error->message);
        g_error_free(error);
    }
    g_free(command);
}

static void
filer_close(const char *dir)
{
    GError *error = NULL;
    char *command = gconf_client_get_string(gconf_client_get_default(),
            "/apps/rox-media/filer_close_command", &error);
    
    if (!command && !error)
        command = g_strdup("rox -D");
    if (command && command[0])
        filer_run(command, dir, NULL);
    if (error)
        g_error_free(error);
    g_free(command);
}

static void
filer_close_device(GduDevice *dev)
{
    const char *path = gdu_device_get_mount_path(dev);
    
    if (path && path[0])
        filer_close(path);
}

static void
mount_complete(GduDevice *device, char *mount_point,
        GError *error, AddedDeviceData *add)
{
    add->mounting = FALSE;
    show_mount_status(add);
    close_menu(add);
    if (error)
    {
        dialog_error(icon_group_get_window(add->media->group), 0,
                _("Unable to mount %s: %s"),
                gdu_device_get_device_file(add->dev), error->message);
        /* FIXME: Are we supposed to free this error? */
        return;
    }
    if (add->open_when_mounted)
    {
        filer_open(add, mount_point);
    }
}

static void
unmount_complete(GduDevice *device, GError *error, AddedDeviceData *add)
{
    if (error)
    {
        dialog_error(icon_group_get_window(add->media->group), 0,
                _("Unable to unmount %s: %s"),
                gdu_device_get_device_file(add->dev), error->message);
        /* FIXME: Are we supposed to free this error? */
    }
    add->mounting = FALSE;
    show_mount_status(add);
    close_menu(add);
    if (add->ejectee)
    {
        gdu_device_op_drive_eject(add->ejectee, NULL, NULL);
        add->ejectee = NULL;
    }
}

static char **get_mount_options(AddedDeviceData *add, const char *ptype)
{
    char *key = g_strdup_printf(
            "/system/storage/default_options/%s/mount_options", ptype);
    GError *error = NULL;
    GSList *list = gconf_client_get_list(gconf_client_get_default(),
            key, GCONF_VALUE_STRING, &error);
    char **options;
    guint n;
    GSList *link;
    
    if (error)
    {
        dialog_error(icon_group_get_window(add->media->group), 0,
                _("GConf error looking up mount options for %s: %s"),
                ptype, error->message);
        g_error_free(error);
        return NULL;
    }
    if (!list)
        return NULL;
    n = g_slist_length(list);
    options = g_new(char *, n + 1);
    for (n = 0, link = list; link; link = g_slist_next(link))
    {
        char *o = link->data;
        
        /* uid and gid get set anyway */
        if (g_str_has_prefix(o, "uid="))
        {
            // options[n++] = g_strdup_printf("uid=%d", getuid());
            g_free(o);
        }
        else if (g_str_has_prefix(o, "gid="))
        {
            // options[n++] = g_strdup_printf("gid=%d", getgid());
            g_free(o);
        }
        else if (!strcmp(o, "locale="))
        {
            options[n++] = g_strdup_printf("locale=%s",
                    setlocale(LC_ALL, NULL));
            g_free(o);
        }
        else if (o[strlen(o) - 1] == '=')
        {
            dialog_warning(icon_group_get_window(add->media->group), 0,
                    _("Unknown mount option for %s: %s"), ptype, o);
            g_free(o);
        }
        else
        {
            options[n++] = o;
        }
    }
    options[n] = NULL;
    g_slist_free(list);
    return options;
}

static void
mount_clicked(AddedDeviceData *add, gpointer widget)
{
    const char *ptype;
    char **options = NULL;
    
    ptype = gdu_device_id_get_type(add->dev);
    if (ptype && ptype[0])
    {
        options = get_mount_options(add, ptype);
    }
    add->mounting = TRUE;
    gdu_device_op_filesystem_mount(add->dev, options,
            (GduDeviceFilesystemMountCompletedFunc) mount_complete,
            add);
    show_mount_status(add);
    g_strfreev(options);
}

static void
unmount_clicked(AddedDeviceData *add, gpointer widget)
{
    filer_close_device(add->dev);
    add->mounting = TRUE;
    show_mount_status(add);
    gdu_device_op_filesystem_unmount(add->dev,
            (GduDeviceFilesystemUnmountCompletedFunc) unmount_complete, add);
}

static void
eject_clicked(AddedDeviceData *add, gpointer widget)
{
    GduDevice *dev;
    
    filer_close_device(add->dev);
    add->mounting = TRUE;
    show_mount_status(add);
    if (gdu_device_drive_get_is_media_ejectable(add->dev))
    {
        dev = add->dev;
    }
    else if (add->odd && gdu_device_drive_get_is_media_ejectable(add->odd->dev))
    {
        dev = add->odd->dev;
    }
    else
    {
        dialog_error(icon_group_get_window(add->media->group), 0,
                _("Device %s somehow lost its ability to eject, "
                "attempting just to unmount it."),
                gdu_device_get_device_file(add->dev));
        gdu_device_op_filesystem_unmount(add->dev,
                (GduDeviceFilesystemUnmountCompletedFunc) unmount_complete,
                add);
        return;
    }
    close_menu(add);
    if (gdu_device_is_mounted(add->dev))
    {
        add->ejectee = dev;
        gdu_device_op_filesystem_unmount(add->dev,
                (GduDeviceFilesystemUnmountCompletedFunc) unmount_complete,
                add);
    }
    else
    {
        gdu_device_op_drive_eject(dev, NULL, NULL);
    }
}

static void
menu_destroyed(AddedDeviceData *add, GtkWidget *menu)
{
    add->menu = NULL;
}

static void
properties_destroyed(GtkWidget *widget, AddedDeviceData *add)
{
    dialog_properties_free(add->pd);
    add->pd = NULL;
}

static void
properties_response(GtkWidget *widget, int response, AddedDeviceData *add)
{
    gtk_widget_destroy(widget);
}

static void
properties_clicked(AddedDeviceData *add, gpointer widget)
{
    GtkWidget *dialog;
    
    add->pd = dialog_properties_new(add->presentable, add->dev);
    dialog = dialog_properties_get_widget(add->pd);
    g_signal_connect(dialog, "destroy", G_CALLBACK(properties_destroyed), add);
    g_signal_connect(dialog, "response", G_CALLBACK(properties_response), add);
    gtk_widget_show_all(dialog);
}

static void
popup_device_menu(AddedDeviceData *add, guint button, guint32 activate_time)
{
    gboolean mounted = gdu_device_is_mounted(add->dev);
    GtkWidget *menu = gtk_menu_new();
    GtkWidget *item = gtk_menu_item_new_with_label(
            mounted ? _("Unmount") : _("Mount"));
    
    g_signal_connect_swapped(item, "activate",
            G_CALLBACK(mounted ? unmount_clicked : mount_clicked), add);
    gtk_widget_set_sensitive(item, !add->mounting);
    gtk_widget_show(item);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
    
    item = gtk_menu_item_new_with_label(_("Eject"));
    if (gdu_device_drive_get_is_media_ejectable(add->dev) ||
            (add->odd &&
            gdu_device_drive_get_is_media_ejectable(add->odd->dev)))
    {
        gtk_widget_set_sensitive(item, TRUE);
        g_signal_connect_swapped(item, "activate",
                G_CALLBACK(eject_clicked), add);
    }
    else
    {
        gtk_widget_set_sensitive(item, FALSE);
    }
    gtk_widget_show(item);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
    
    item = gtk_menu_item_new_with_label(_("Properties"));
    g_signal_connect_swapped(item, "activate",
            G_CALLBACK(properties_clicked), add);
    gtk_widget_show(item);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
    
    g_signal_connect_swapped(menu, "destroy", G_CALLBACK(menu_destroyed), add);
    add->menu = menu;
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
            h_panel_icon_get_menu_position_func(add->panel_icon),
            add->panel_icon, button, activate_time);
}

static gboolean
device_click_handler(HPanelIcon *icon, GdkEventButton *event,
        AddedDeviceData *add)
{
    if (event->type == GDK_BUTTON_PRESS)
    {
        if (event->button == 1 && !add->mounting)
        {
            if (gdu_device_is_mounted(add->dev))
            {
                filer_open(add, gdu_device_get_mount_path(add->dev));
            }
            else
            {
                add->open_when_mounted = TRUE;
                mount_clicked(add, icon);
            }
            return TRUE;
        }
        else if (event->button == 3)
        {
            popup_device_menu(add, event->button, event->time);
            return TRUE;
        }
    }
    return FALSE;
}

static void
check_add_presentable(GduPool *pool, GduPresentable *presentable,
        RoxMedia *media)
{
    GduDevice *dev = gdu_presentable_get_device(presentable);
    gboolean interesting = FALSE;
    
    if (!dev)
    {
        return;
    }
    if (GDU_IS_VOLUME(presentable) &&
            !g_strcmp0(gdu_device_id_get_usage(dev), "filesystem"))
    {
        if (gdu_device_is_removable(dev))
        {
            interesting = TRUE;
        }
        else
        {
            /* Partitions aren't flagged removable, so check top level drive */
            GduDrive *drive = gdu_volume_get_drive(GDU_VOLUME(presentable));
            GduDevice *drive_dev = gdu_presentable_get_device(
                    GDU_PRESENTABLE(drive));
        
            if (drive_dev)
            {
                interesting = gdu_device_is_removable(drive_dev);
                g_object_unref(drive_dev);
            }
            g_object_unref(drive);
        }
    }
    if (interesting)
    {
        AddedDeviceData *add = g_new(AddedDeviceData, 1);
        GIcon *gicon;
        
        add->media = media;
        add->presentable = presentable;
        add->dev = dev;
        add->panel_icon = NULL;
        add->odd = NULL;
        add->mounting = FALSE;
        add->emblem_status = EMBLEM_NONE;
        add->menu = NULL;
        add->open_when_mounted = FALSE;
        add->ejectee = NULL;
        add->pd = NULL;
        gicon = gdu_presentable_get_icon(add->presentable);
        add->icon_source = disable_emblems ?
                (HIconSourceEmblem *) h_icon_source_gicon_new(
                    add->media->screen, gicon) :
                h_icon_source_emblem_new(
                    H_ICON_SOURCE(h_icon_source_gicon_new(add->media->screen,
                                gicon)), NULL);
        show_mount_status(add);
        add->panel_icon = icon_group_add_icon(add->media->group,
                H_ICON_SOURCE(add->icon_source));
        g_signal_connect(add->panel_icon, "button-press-event",
                G_CALLBACK(device_click_handler), add);
        g_object_unref(gicon);
        add->removed_tag = g_signal_connect(add->dev, "removed",
                G_CALLBACK(device_removed), add);
        add->changed_tag = g_signal_connect(add->dev, "changed",
                G_CALLBACK(device_changed), add);
        set_tooltip(add);
        if (gdu_device_is_optical_disc(add->dev))
        {
            /* Don't seem to get "removed" signals on optical discs
             * so use parent drives to keep track of them.
             */
            OpticalDriveData *odd = 
                    add->odd = g_new(OpticalDriveData, 1);
            GduPresentable *tl_pres =
                    gdu_presentable_get_toplevel(add->presentable);
            GduDevice *tl_dev = odd->dev = gdu_presentable_get_device(tl_pres);
            
            odd->disc_add = add;
            odd->removed_tag = g_signal_connect(tl_dev, "removed",
                    G_CALLBACK(parent_removed), odd);
            odd->changed_tag = g_signal_connect(tl_dev, "changed",
                    G_CALLBACK(parent_changed), odd);
            g_object_unref(tl_pres);
        }
        if (!gdu_device_is_mounted(dev))
        {
            add->open_when_mounted =
                    gconf_client_get_bool(gconf_client_get_default(),
                            "/apps/rox-media/autoopen", NULL);
            if (gconf_client_get_bool(gconf_client_get_default(),
                            "/apps/rox-media/automount", NULL))
            {
                mount_clicked(add, NULL);
            }
        }
    }
    else
    {
        g_object_unref(dev);
    }
}

static void
preferences_clicked(GtkWidget *w, IconGroup *group)
{
    const char *args[2];
    GError *error = NULL;

    args[0] = appdir_get_configlet();
    args[1] = NULL;
    if (!g_spawn_async(NULL, (char **) args, NULL, 0,
            NULL, NULL, NULL, &error))
    {
        dialog_error(icon_group_get_window(group), 0,
                _("Unable to run rox-media-config: %s"),
                error->message);
        g_error_free(error);
    }
}

static void
popup_main_menu(RoxMedia *media, HPanelIcon *icon,
        guint button, guint evtime)
{
    GtkWidget *menu = gtk_menu_new();
    GtkMenuShell *shell = GTK_MENU_SHELL(menu);
    GtkWidget *item;
    
    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
    g_signal_connect(item, "activate", G_CALLBACK(dialog_about), NULL);
    gtk_widget_show(item);
    gtk_menu_shell_append(shell, item);
    
    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
    g_signal_connect(item, "activate", G_CALLBACK(preferences_clicked),
            media->group);
    gtk_widget_show(item);
    gtk_menu_shell_append(shell, item);
    
    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
    g_signal_connect(item, "activate", G_CALLBACK(gtk_main_quit), NULL);
    gtk_widget_show(item);
    gtk_menu_shell_append(shell, item);
    
    if (!media->systray)
        gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(icon), NULL);
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
            h_panel_icon_get_menu_position_func(icon),
            icon, button, evtime);
}

static gboolean
main_icon_click_handler(HPanelIcon *icon,
        GdkEventButton *event, RoxMedia *media)
{
    if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
    {
        popup_main_menu(media, icon, event->button, event->time);
        return TRUE;
    }
    return FALSE;
}

static void
main_icon_rox_popup_handler(HPanelIcon *icon, RoxMedia *media)
{
    popup_main_menu(media, icon, 0, gtk_get_current_event_time());
}

static void
main_icon_systray_popup_handler(HPanelIcon *icon,
        guint button, guint evtime, RoxMedia *media)
{
    popup_main_menu(media, icon, button, evtime);
}

static void
init_group(IconGroup *group, gboolean systray)
{
    RoxMedia *media = g_new(RoxMedia, 1);
    GList *presentables, *link;
    GduPool *pool;
    
    media->group = group;
    media->main_icon = icon_group_add_icon(group,
            H_ICON_SOURCE(h_icon_source_filename_new(appdir_get_app_icon())));
    media->systray = systray;
    if (systray)
    {
        if (!gtk_status_icon_get_geometry(GTK_STATUS_ICON(media->main_icon),
                &media->screen, NULL, NULL))
        {
            media->screen = gdk_screen_get_default();
        }
    }
    else
    {
        media->screen = gtk_widget_get_screen(GTK_WIDGET(media->main_icon));
        if (!media->screen)
            media->screen = gdk_screen_get_default();
    }
    //media->added_devices = NULL;
    g_signal_connect(media->main_icon, "button-press-event",
            G_CALLBACK(main_icon_click_handler), media);
    g_signal_connect(media->main_icon, "popup-menu",
            systray ? G_CALLBACK(main_icon_systray_popup_handler) :
                      G_CALLBACK(main_icon_rox_popup_handler),
            media);
    
    pool = media->pool = gdu_pool_new();
    presentables = gdu_pool_get_presentables(pool);
    for (link = presentables; link; link = g_list_next(link))
        check_add_presentable(pool, GDU_PRESENTABLE(link->data), media);
    g_list_free(presentables);
    g_signal_connect(pool, "presentable-added",
            G_CALLBACK(check_add_presentable), media);
    icon_group_show(group);
}
    
static char *pidfile = NULL;

static void
remove_pidfile(void)
{
    g_unlink(pidfile);
}

int
main(int argc, char **argv)
{
    int panel_id = -1;
    IconGroup *group;
    GError *error = NULL;
    GOptionEntry entries[] = {
        { "appdir", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME,
                &appdir, NULL, NULL },
        { "panel-id", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_INT,
                &panel_id, NULL, NULL },
        { "pidfile", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME,
                &pidfile, NULL, NULL },
        { "disable-emblems", 0, 0, G_OPTION_ARG_NONE,
                &disable_emblems,
                _("Disable emblems (workaround for gdk-pixbuf composite bug)"),
                NULL },
        { NULL }
    };
    GOptionContext *context;
    FILE *fp;
    
    gtk_init(&argc, &argv);
    context = g_option_context_new(N_(" - removable media handler for ROX"));
    g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
    g_option_context_add_group(context, gtk_get_option_group(TRUE));
    if (!g_option_context_parse (context, &argc, &argv, &error))
    {
        (void) dialog_error(NULL, 0,
                _("Error parsing command line options: %s"), error->message);
        exit(1);
    }
    if (!pidfile)
    {
        (void) dialog_error(NULL, 0, _("pidfile not specified. "
                "Perhaps you tried to run rox-media's binary directly "
                "instead of via its script."));
        exit(1);
    }
    
    fp = fopen(pidfile, "w");
    if (fp)
    {
        fprintf(fp, "%d", getpid());
        fclose(fp);
        atexit(remove_pidfile);
    }
    else
    {
        (void) dialog_error(NULL, 0, _("Unable to create pidfile %s: %s."),
                pidfile, strerror(errno));
        exit(1);
    }
    
    if (panel_id != -1)
        group = icon_group_new_for_rox_panel(panel_id);
    else
        group = icon_group_new_for_systray();
    init_group(group, panel_id == -1);
    icon_group_show_all(group);
    
    gtk_main();
    
    return 0;
}
